###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.16 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import calendar_event
import base_item
import utils
import todo

calendar_day_add_redirect = HTMLFile('calendar_day_add_redirect', globals())
manage_add_calendar_day_form = HTMLFile('calendar_day_add', globals())


def manage_add_calendar_day(self, id, title, REQUEST=None):
    "Add an instance of the calendar_day class"

    self._setObject(id, calendar_day(id, title))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class calendar_day(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'CalendarDay'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
		{'name':'CalendarEvent', 'action':'manage_add_calendar_event_form'},
		{'name':'Todo', 'action':'manage_add_todo_form'},
		)

	manage_add_calendar_event_form = calendar_event.manage_add_calendar_event_form
	manage_add_calendar_event = calendar_event.manage_add_calendar_event
	calendar_event_add_redirect = calendar_event.calendar_event_add_redirect

	manage_add_todo_form = todo.manage_add_todo_form
	manage_add_todo = todo.manage_add_todo

	make_hour_minutes_select = utils.make_hour_minutes_select

	day_start = 3600 * 6 # The hour we start the day at
	day_end = 3600 * 21 # The hour we end the day at

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add CalendarEvent',
		 ('manage_add_calendar_event', 'manage_add_calendar_event_form')),
		('View CalendarDays',
		 ('',)),
		)

	index_html = HTMLFile('calendar_day_index', globals())
	display_week = HTMLFile('calendar_day_display_week', globals())
	navigate = HTMLFile('calendar_day_navigate', globals())
	display = HTMLFile('calendar_day_display', globals())


	def __init__(self, id, title=''):
		"Initialize the class instance"

		self.id = id
		self.title = title


	def get_todos(self):
		"""Returns a list of todos."""

		return self.objectValues('Todo')


	def get_todos_pending(self):
		"""Returns a list of todos not completed."""

		todos = self.get_todos()

		return filter(lambda x: x.status == 1, todos)


	def get_events(self):
		"""Returns a list of events."""

		return self.objectValues('CalendarEvent')


	def get_time_groups(self):
		"""Returns a list of time groups."""

		time_groups = []

		for hour in range(self.day_start, self.day_end, 3600):

			time_groups.append(
				{'time':hour,
				'time_formatted':utils.format_time(hour)
				})

		return time_groups


	def get_events_pending_for_time_group(self, time_start, time_end):
		"""Returns a list of events pending and starting in the given time group."""

		return self.get_events_for_time_group(time_start, time_end, all=None)


	def get_events_for_time_group(self, time_start, time_end, all=1):
		"""Returns a list of events starting in the given time group."""

		events = []

		for event in self.get_events():

			if event.start >= time_start and event.start < time_end:

				events.append(event)

		if not all:

			events = filter(lambda x: x.status == 1, events)

		return events


	def get_hour_grouped_events(self):
		"""Returns a list of grouped events."""


		grouped_events = []
		contained_events = self.get_events()

		for hour in range(self.day_start, self.day_end, 3600):

			next_hour = hour + 3600
			events = [{'time':utils.format_time(hour),'events':[]}]

			for event in contained_events:

				if event.start >= hour and event.start < next_hour:

					events['events'].append(event)
					contained_events.remove(event)

			grouped_events.append(events)

		return grouped_events


	def get_week_days(self):
		"""Returns the week days surrounding the day."""

		return self.get_month().get_week_days(int(self.id))


	def get_previous_day(self):
		"""Returns the previous day."""

		if self.id == '01':

			# We need to look in the previous month

			previous_month = self.get_month().get_previous_month()
			previous_day = previous_month.get_last_day()

		else:

			previous_day = self.get_month()[utils.pad_integer(int(self.id) - 1)]

		return previous_day


	def get_next_day(self):
		"""Returns the next day."""

		if self.id == self.get_month().get_last_day().id:

			# We need to look in the next month

			next_month = self.get_month().get_next_month()
			next_day = next_month.get_first_day()

		else:

			next_day = self.get_month()[utils.pad_integer(int(self.id) + 1)]

		return next_day


	def get_day(self):
		"""Returns self."""

		return self


	def get_context_day(self):

		return self


	def get_context_day_id(self):

		return self.id


	def get_date(self, reverse=None):
		"""Returns a tuple indicating year, month and day."""

		date = list(
			(self.getParentNode().getParentNode().id,
			self.getParentNode().id,
			self.id))

		if reverse:

			date.reverse()

		return tuple(date)


	def render_date(self, reverse=None):

		return "%s/%s/%s" % self.get_date(reverse=reverse)


	def old_display(self, manage=None):
		"""Returns an HTML formatted output of the current day and events."""

		todos = self.objectValues('Todo')
		events = self.objectValues('CalendarEvent')
		date = list(self.get_date())
		date.reverse()
		date_string = "%s/%s/%s" % tuple(date)

		display_html = """<table border="0" cellpadding="%s" cellspacing="%s"
		valign="top" bgcolor="%s" width="100%%">
		<tr bgcolor="%s"><th colspan="3" align="left"><a href="%s">Events and todos for %s</a></th></tr>
		<tr bgcolor="%s"><th width="10%%">Time</th><th width="80%%">Title</th><th width="10%%">Priority</th></tr>
		""" % \
		(
			2,
			self.table_cellspacing0,
			self.color_body1,
			self.color_header1,
			self.relative_url(),
			self.title + ' ' + date_string,
			self.color_header0
		)

		if todos:

			todo_table_html = """<tr><td valign="top">N/A</td><td colspan="2"><table border="0" width="100%">"""

			for todo in todos:

				todo_table_html = todo_table_html + """<tr>
					<td align="left"><input type="checkbox" name="ids:list" value="%s"><a href="%s">%s</a></td><td align="right">%s</td></tr>""" % \
					(todo.id, todo.relative_url(), todo.title, todo.render_priority())

			else:

				todo_table_html = todo_table_html + """<tr><td colspan="2"></td></tr>"""

			display_html = display_html + todo_table_html + "</table></td></tr>"

		for hour in range(self.day_start, self.day_end, 3600):

			next_hour = hour + 3600

			event_table_html = """<table border="0" width="100%">"""

			# Gross hack

			event_table_html_len = len(event_table_html)

			event_count = 1

			for event in events:

				if event.start >= hour and event.start < next_hour:

					if event_count % 2:

						bgcolor = self.color_sequence_even0

					else:

						bgcolor = self.color_sequence_odd0

					event_table_html = event_table_html + """<tr bgcolor="%s"><td align="left"><input type="checkbox" name="ids:list" value="%s"><a href="%s">%s</a></td><td align="right">%s</td></tr>""" % \
						(bgcolor, event.id, event.relative_url(), "(%s - %s) " % (event.render_start(), event.render_end()) + event.title, event.render_priority())

					event_count = event_count + 1

			else:

				# Bleh FIXME

				if len(event_table_html) == event_table_html_len:

					event_table_html = event_table_html + \
						"""<tr><td colspan="2">&nbsp;</td></tr></table>"""

				else:

					event_table_html = event_table_html + "</table>"

			# We reverse the colors here, so that it's easier to see.

			if (hour / 3600) % 2:

				bgcolor = self.color_sequence_odd0

			else:

				bgcolor = self.color_sequence_even0

			display_html = display_html + """<tr bgcolor="%s"><td valign="top" align="left"> <a href="%s/manage_add_calendar_event_form?start_time=%s">%s:00</a></td><td colspan="2">%s</td></tr>""" % \
					(bgcolor, self.relative_url(), hour, (hour / 3600), event_table_html)

		return display_html + """<tr><td colspan="2"><input type="submit" value=" Finish " name="%s/manage_finish:method"><input type="submit" value=" Delete " name="%s/manage_delete:method"></td></tr></table>""" % (self.relative_url(), self.relative_url())


	def old_display(self, manage=1):
		"""Returns an HTML formatted output of the current day and events."""

		events = self.objectValues('CalendarEvent')
		events.sort(lambda x,y: cmp(x.start, y.start))

		if not events:

			return """
				<table border="0" width="100%%" bgcolor="%s">
				<tr>
					<td align="left" colspan="2">
						<a href="%s"><h3>No events for %s</h3></a>
					</td>
				</tr>
				</table""" % (self.color_body1, self.absolute_url(), self.title)

		tmp = """<table border="0" cellpadding="%s" cellspacing="%s"
		valign="top" bgcolor="%s" width="100%%">
		<tr bgcolor="%s"><th colspan="3" align="left"><a href="%s">Events and todos for %s</a></th></tr>
		<tr bgcolor="%s"><th>Time</th><th>Title</th><th>Priority</th></tr>
		""" % \
		(self.table_cellpadding0,
			self.table_cellspacing0,
			self.color_body1,
			self.color_header1,
			self.relative_url(),
			self.title,
			self.color_header0
		)

		for calendar_event in range(len(events)):

			if calendar_event % 2:

				bgcolor = self.color_sequence_even0

			else:

				bgcolor = self.color_sequence_odd0

			tmp = tmp + """
			<tr bgcolor="%s">
			<td width="25%%" valign="top"><a href="%s">%s - %s</a>&nbsp;</td>
			<td valign="top">%s %s&nbsp;</td>
			<td valign="top">%s&nbsp;</td>
			</tr>""" % \
				(bgcolor,
				events[calendar_event].absolute_url(),
				utils.format_time(events[calendar_event].start),
				utils.format_time(events[calendar_event].end),
				events[calendar_event].title,
				events[calendar_event].render_alarm(),
				events[calendar_event].render_priority())

		else:

			if len(events) == 0:

				tmp = tmp + """<tr><td colspan="3">None.</td></tr>"""

		tmp = tmp + """</table>"""

		return tmp


	def manage_complete(self, REQUEST, RESPONSE):
		"""Updates the todo or event status(es) to completed."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				self[id].status = 0

		RESPONSE.redirect(REQUEST['__referrer'])


	def manage_mute(self, REQUEST, RESPONSE):
		"""Mutes the given calendar events."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				object = self[id]

				if object.meta_type == 'CalendarEvent':

					object.alarm_muted = 1

		RESPONSE.redirect(REQUEST['__referrer'])


	def manage_snooze(self, REQUEST, RESPONSE):
		"""Sets the alarm n seconds forward."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				object = self[id]

				if object.meta_type == 'CalendarEvent':


					object.snooze(self.snooze)


		RESPONSE.redirect(REQUEST['__referrer'])


default__class_init__(calendar_day)

